@@ -15,7 +15,6 @@ directory.
 
 import ast
 from functools import lru_cache, reduce
-import logging
 from numbers import Number
 import operator
 import os
@@ -23,25 +22,27 @@ import re
 
 import numpy as np
 
-from matplotlib import animation, cbook
+from matplotlib import _api, cbook
 from matplotlib.cbook import ls_mapper
-from matplotlib.fontconfig_pattern import parse_fontconfig_pattern
-from matplotlib.colors import is_color_like
+from matplotlib.colors import Colormap, is_color_like
+from matplotlib._fontconfig_pattern import parse_fontconfig_pattern
+from matplotlib._enums import JoinStyle, CapStyle
 
 # Don't let the original cycler collide with our validating cycler
 from cycler import Cycler, cycler as ccycler
 
 
-_log = logging.getLogger(__name__)
 # The capitalized forms are needed for ipython at present; this may
 # change for later versions.
-interactive_bk = ['GTK3Agg', 'GTK3Cairo',
-                  'MacOSX',
-                  'nbAgg',
-                  'Qt4Agg', 'Qt4Cairo', 'Qt5Agg', 'Qt5Cairo',
-                  'TkAgg', 'TkCairo',
-                  'WebAgg',
-                  'WX', 'WXAgg', 'WXCairo']
+interactive_bk = [
+    'GTK3Agg', 'GTK3Cairo', 'GTK4Agg', 'GTK4Cairo',
+    'MacOSX',
+    'nbAgg',
+    'QtAgg', 'QtCairo', 'Qt5Agg', 'Qt5Cairo',
+    'TkAgg', 'TkCairo',
+    'WebAgg',
+    'WX', 'WXAgg', 'WXCairo',
+]
 non_interactive_bk = ['agg', 'cairo',
                       'pdf', 'pgf', 'ps', 'svg', 'template']
 all_backends = interactive_bk + non_interactive_bk
@@ -65,9 +66,9 @@ class ValidateInStrings:
     def __call__(self, s):
         if self._deprecated_since:
             name, = (k for k, v in globals().items() if v is self)
-            cbook.warn_deprecated(
+            _api.warn_deprecated(
                 self._deprecated_since, name=name, obj_type="function")
-        if self.ignorecase:
+        if self.ignorecase and isinstance(s, str):
             s = s.lower()
         if s in self.valid:
             return self.valid[s]
@@ -136,17 +137,6 @@ def _validate_date(s):
             f'{s!r} should be a string that can be parsed by numpy.datetime64')
 
 
-@cbook.deprecated("3.2", alternative="os.path.exists")
-def validate_path_exists(s):
-    """If s is a path, return s, else False"""
-    if s is None:
-        return None
-    if os.path.exists(s):
-        return s
-    else:
-        raise RuntimeError('"%s" should be a path but it does not exist' % s)
-
-
 def validate_bool(b):
     """Convert b to ``bool`` or raise."""
     if isinstance(b, str):
@@ -156,45 +146,7 @@ def validate_bool(b):
     elif b in ('f', 'n', 'no', 'off', 'false', '0', 0, False):
         return False
     else:
-        raise ValueError('Could not convert "%s" to bool' % b)
-
-
-@cbook.deprecated("3.3")
-def validate_bool_maybe_none(b):
-    """Convert b to ``bool`` or raise, passing through *None*."""
-    if isinstance(b, str):
-        b = b.lower()
-    if b is None or b == 'none':
-        return None
-    if b in ('t', 'y', 'yes', 'on', 'true', '1', 1, True):
-        return True
-    elif b in ('f', 'n', 'no', 'off', 'false', '0', 0, False):
-        return False
-    else:
-        raise ValueError('Could not convert "%s" to bool' % b)
-
-
-def _validate_tex_preamble(s):
-    if s is None or s == 'None':
-        cbook.warn_deprecated(
-            "3.3", message="Support for setting the 'text.latex.preamble' or "
-            "'pgf.preamble' rcParam to None is deprecated since %(since)s and "
-            "will be removed %(removal)s; set it to an empty string instead.")
-        return ""
-    try:
-        if isinstance(s, str):
-            return s
-        elif np.iterable(s):
-            cbook.warn_deprecated(
-                "3.3", message="Support for setting the 'text.latex.preamble' "
-                "or 'pgf.preamble' rcParam to a list of strings is deprecated "
-                "since %(since)s and will be removed %(removal)s; set it to a "
-                "single string instead.")
-            return '\n'.join(s)
-        else:
-            raise TypeError
-    except TypeError as e:
-        raise ValueError('Could not convert "%s" to string' % s) from e
+        raise ValueError(f'Cannot convert {b!r} to bool')
 
 
 def validate_axisbelow(s):
@@ -204,14 +156,8 @@ def validate_axisbelow(s):
         if isinstance(s, str):
             if s == 'line':
                 return 'line'
-            if s.lower().startswith('line'):
-                cbook.warn_deprecated(
-                    "3.3", message=f"Support for setting axes.axisbelow to "
-                    f"{s!r} to mean 'line' is deprecated since %(since)s and "
-                    f"will be removed %(removal)s; set it to 'line' instead.")
-                return 'line'
-    raise ValueError('%s cannot be interpreted as'
-                     ' True, False, or "line"' % s)
+    raise ValueError(f'{s!r} cannot be interpreted as'
+                     ' True, False, or "line"')
 
 
 def validate_dpi(s):
@@ -235,9 +181,11 @@ def _make_type_validator(cls, *, allow_none=False):
         if (allow_none and
                 (s is None or isinstance(s, str) and s.lower() == "none")):
             return None
+        if cls is str and not isinstance(s, str):
+            raise ValueError(f'Could not convert {s!r} to str')
         try:
             return cls(s)
-        except ValueError as e:
+        except (TypeError, ValueError) as e:
             raise ValueError(
                 f'Could not convert {s!r} to {cls.__name__}') from e
 
@@ -261,6 +209,15 @@ validate_floatlist = _listify_validator(
     validate_float, doc='return a list of floats')
 
 
+def _validate_pathlike(s):
+    if isinstance(s, (str, os.PathLike)):
+        # Store value as str because savefig.directory needs to distinguish
+        # between "" (cwd) and "." (cwd, but gets updated by user selections).
+        return os.fsdecode(s)
+    else:
+        return validate_string(s)
+
+
 def validate_fonttype(s):
     """
     Confirm that this is a Postscript or PDF font type that we know how to
@@ -296,45 +253,14 @@ def validate_backend(s):
     return backend
 
 
-validate_toolbar = ValidateInStrings(
-    'toolbar', ['None', 'toolbar2', 'toolmanager'], ignorecase=True,
-    _deprecated_since="3.3")
-
-
-@cbook.deprecated("3.3")
-def _make_nseq_validator(cls, n=None, allow_none=False):
-
-    def validator(s):
-        """Convert *n* objects using ``cls``, or raise."""
-        if isinstance(s, str):
-            s = [x.strip() for x in s.split(',')]
-            if n is not None and len(s) != n:
-                raise ValueError(
-                    f'Expected exactly {n} comma-separated values, '
-                    f'but got {len(s)} comma-separated values: {s}')
-        else:
-            if n is not None and len(s) != n:
-                raise ValueError(
-                    f'Expected exactly {n} values, '
-                    f'but got {len(s)} values: {s}')
-        try:
-            return [cls(val) if not allow_none or val is not None else val
-                    for val in s]
-        except ValueError as e:
-            raise ValueError(
-                f'Could not convert all entries to {cls.__name__}s') from e
-
-    return validator
-
-
-@cbook.deprecated("3.3")
-def validate_nseq_float(n):
-    return _make_nseq_validator(float, n)
-
-
-@cbook.deprecated("3.3")
-def validate_nseq_int(n):
-    return _make_nseq_validator(int, n)
+def _validate_toolbar(s):
+    s = ValidateInStrings(
+        'toolbar', ['None', 'toolbar2', 'toolmanager'], ignorecase=True)(s)
+    if s == 'toolmanager':
+        _api.warn_external(
+            "Treat the new Tool classes introduced in v1.5 as experimental "
+            "for now; the API and rcParam may change in future versions.")
+    return s
 
 
 def validate_color_or_inherit(s):
@@ -357,6 +283,27 @@ def validate_color_for_prop_cycle(s):
     return validate_color(s)
 
 
+def _validate_color_or_linecolor(s):
+    if cbook._str_equal(s, 'linecolor'):
+        return s
+    elif cbook._str_equal(s, 'mfc') or cbook._str_equal(s, 'markerfacecolor'):
+        return 'markerfacecolor'
+    elif cbook._str_equal(s, 'mec') or cbook._str_equal(s, 'markeredgecolor'):
+        return 'markeredgecolor'
+    elif s is None:
+        return None
+    elif isinstance(s, str) and len(s) == 6 or len(s) == 8:
+        stmp = '#' + s
+        if is_color_like(stmp):
+            return stmp
+        if s.lower() == 'none':
+            return None
+    elif is_color_like(s):
+        return s
+
+    raise ValueError(f'{s!r} does not look like a color arg')
+
+
 def validate_color(s):
     """Return a valid color arg."""
     if isinstance(s, str):
@@ -384,8 +331,11 @@ def validate_color(s):
 
 validate_colorlist = _listify_validator(
     validate_color, allow_stringlist=True, doc='return a list of colorspecs')
-validate_orientation = ValidateInStrings(
-    'orientation', ['landscape', 'portrait'], _deprecated_since="3.3")
+
+
+def _validate_cmap(s):
+    _api.check_isinstance((str, Colormap), cmap=s)
+    return s
 
 
 def validate_aspect(s):
@@ -434,28 +384,25 @@ def validate_fontweight(s):
         raise ValueError(f'{s} is not a valid font weight.') from e
 
 
+def validate_fontstretch(s):
+    stretchvalues = [
+        'ultra-condensed', 'extra-condensed', 'condensed', 'semi-condensed',
+        'normal', 'semi-expanded', 'expanded', 'extra-expanded',
+        'ultra-expanded']
+    # Note: Historically, stretchvalues have been case-sensitive in Matplotlib
+    if s in stretchvalues:
+        return s
+    try:
+        return int(s)
+    except (ValueError, TypeError) as e:
+        raise ValueError(f'{s} is not a valid font stretch.') from e
+
+
 def validate_font_properties(s):
     parse_fontconfig_pattern(s)
     return s
 
 
-def _validate_mathtext_fallback_to_cm(b):
-    """
-    Temporary validate for fallback_to_cm, while deprecated
-
-    """
-    if isinstance(b, str):
-        b = b.lower()
-    if b is None or b == 'none':
-        return None
-    else:
-        cbook.warn_deprecated(
-            "3.3", message="Support for setting the 'mathtext.fallback_to_cm' "
-            "rcParam is deprecated since %(since)s and will be removed "
-            "%(removal)s; use 'mathtext.fallback : 'cm' instead.")
-        return validate_bool_maybe_none(b)
-
-
 def _validate_mathtext_fallback(s):
     _fallback_fonts = ['cm', 'stix', 'stixsans']
     if isinstance(s, str):
@@ -471,71 +418,15 @@ def _validate_mathtext_fallback(s):
             "fallback off.")
 
 
-validate_fontset = ValidateInStrings(
-    'fontset',
-    ['dejavusans', 'dejavuserif', 'cm', 'stix', 'stixsans', 'custom'],
-    _deprecated_since="3.3")
-validate_mathtext_default = ValidateInStrings(
-    'default', "rm cal it tt sf bf default bb frak scr regular".split(),
-    _deprecated_since="3.3")
-_validate_alignment = ValidateInStrings(
-    'alignment',
-    ['center', 'top', 'bottom', 'baseline', 'center_baseline'],
-    _deprecated_since="3.3")
-
-
 def validate_whiskers(s):
-    if s == 'range':
-        cbook.warn_deprecated(
-            "3.2", message="Support for setting the boxplot.whiskers rcParam "
-            "to 'range' is deprecated since %(since)s and will be removed "
-            "%(removal)s; set it to 0, 100 instead.")
-        return 'range'
-    else:
+    try:
+        return _listify_validator(validate_float, n=2)(s)
+    except (TypeError, ValueError):
         try:
-            return _listify_validator(validate_float, n=2)(s)
-        except (TypeError, ValueError):
-            try:
-                return float(s)
-            except ValueError as e:
-                raise ValueError("Not a valid whisker value ['range', float, "
-                                 "(float, float)]") from e
-
-
-@cbook.deprecated("3.2")
-def update_savefig_format(value):
-    # The old savefig.extension could also have a value of "auto", but
-    # the new savefig.format does not.  We need to fix this here.
-    value = validate_string(value)
-    if value == 'auto':
-        cbook.warn_deprecated(
-            "3.2", message="Support for setting the 'savefig.format' rcParam "
-            "to 'auto' is deprecated since %(since)s and will be removed "
-            "%(removal)s; set it to 'png' instead.")
-        value = 'png'
-    return value
-
-
-# Replace by validate_string once deprecation period passes.
-def _update_savefig_format(value):
-    # The old savefig.extension could also have a value of "auto", but
-    # the new savefig.format does not.  We need to fix this here.
-    value = validate_string(value)
-    if value == 'auto':
-        cbook.warn_deprecated(
-            "3.2", message="Support for setting the 'savefig.format' rcParam "
-            "to 'auto' is deprecated since %(since)s and will be removed "
-            "%(removal)s; set it to 'png' instead.")
-        value = 'png'
-    return value
-
-
-validate_ps_papersize = ValidateInStrings(
-    'ps_papersize',
-    ['auto', 'letter', 'legal', 'ledger',
-     'a0', 'a1', 'a2', 'a3', 'a4', 'a5', 'a6', 'a7', 'a8', 'a9', 'a10',
-     'b0', 'b1', 'b2', 'b3', 'b4', 'b5', 'b6', 'b7', 'b8', 'b9', 'b10',
-     ], ignorecase=True, _deprecated_since="3.3")
+            return float(s)
+        except ValueError as e:
+            raise ValueError("Not a valid whisker value [float, "
+                             "(float, float)]") from e
 
 
 def validate_ps_distiller(s):
@@ -575,63 +466,27 @@ def _validate_linestyle(ls):
         # nonsensically interpreted as sequences of numbers (codepoints).
         return np.iterable(x) and not isinstance(x, (str, bytes, bytearray))
 
-    # (offset, (on, off, on, off, ...))
-    if (_is_iterable_not_string_like(ls)
-            and len(ls) == 2
-            and isinstance(ls[0], (type(None), Number))
-            and _is_iterable_not_string_like(ls[1])
-            and len(ls[1]) % 2 == 0
-            and all(isinstance(elem, Number) for elem in ls[1])):
-        if ls[0] is None:
-            cbook.warn_deprecated(
-                "3.3", message="Passing the dash offset as None is deprecated "
-                "since %(since)s and support for it will be removed "
-                "%(removal)s; pass it as zero instead.")
-            ls = (0, ls[1])
-        return ls
-    # For backcompat: (on, off, on, off, ...); the offset is implicitly None.
-    if (_is_iterable_not_string_like(ls)
-            and len(ls) % 2 == 0
-            and all(isinstance(elem, Number) for elem in ls)):
-        return (0, ls)
+    if _is_iterable_not_string_like(ls):
+        if len(ls) == 2 and _is_iterable_not_string_like(ls[1]):
+            # (offset, (on, off, on, off, ...))
+            offset, onoff = ls
+        else:
+            # For backcompat: (on, off, on, off, ...); the offset is implicit.
+            offset = 0
+            onoff = ls
+
+        if (isinstance(offset, Number)
+                and len(onoff) % 2 == 0
+                and all(isinstance(elem, Number) for elem in onoff)):
+            return (offset, onoff)
+
     raise ValueError(f"linestyle {ls!r} is not a valid on-off ink sequence.")
 
 
-def _deprecate_case_insensitive_join_cap(s):
-    s_low = s.lower()
-    if s != s_low:
-        if s_low in ['miter', 'round', 'bevel']:
-            cbook.warn_deprecated(
-                "3.3", message="Case-insensitive capstyles are deprecated "
-                "since %(since)s and support for them will be removed "
-                "%(removal)s; please pass them in lowercase.")
-        elif s_low in ['butt', 'round', 'projecting']:
-            cbook.warn_deprecated(
-                "3.3", message="Case-insensitive joinstyles are deprecated "
-                "since %(since)s and support for them will be removed "
-                "%(removal)s; please pass them in lowercase.")
-        # Else, error out at the check_in_list stage.
-    return s_low
-
-
-def validate_joinstyle(s):
-    s = _deprecate_case_insensitive_join_cap(s)
-    cbook._check_in_list(['miter', 'round', 'bevel'], joinstyle=s)
-    return s
-
-
-def validate_capstyle(s):
-    s = _deprecate_case_insensitive_join_cap(s)
-    cbook._check_in_list(['butt', 'round', 'projecting'], capstyle=s)
-    return s
-
-
 validate_fillstyle = ValidateInStrings(
     'markers.fillstyle', ['full', 'left', 'right', 'bottom', 'top', 'none'])
 
 
-validate_joinstylelist = _listify_validator(validate_joinstyle)
-validate_capstylelist = _listify_validator(validate_capstyle)
 validate_fillstylelist = _listify_validator(validate_fillstyle)
 
 
@@ -641,14 +496,11 @@ def validate_markevery(s):
 
     Parameters
     ----------
-    s : None, int, float, slice, length-2 tuple of ints,
-        length-2 tuple of floats, list of ints
+    s : None, int, (int, int), slice, float, (float, float), or list[int]
 
     Returns
     -------
-    None, int, float, slice, length-2 tuple of ints,
-        length-2 tuple of floats, list of ints
-
+    None, int, (int, int), slice, float, (float, float), or list[int]
     """
     # Validate s against type slice float int and None
     if isinstance(s, (slice, float, int, type(None))):
@@ -674,68 +526,6 @@ def validate_markevery(s):
 
 validate_markeverylist = _listify_validator(validate_markevery)
 
-validate_legend_loc = ValidateInStrings(
-    'legend_loc',
-    ['best',
-     'upper right',
-     'upper left',
-     'lower left',
-     'lower right',
-     'right',
-     'center left',
-     'center right',
-     'lower center',
-     'upper center',
-     'center'], ignorecase=True, _deprecated_since="3.3")
-
-validate_svg_fonttype = ValidateInStrings(
-    'svg.fonttype', ['none', 'path'], _deprecated_since="3.3")
-
-
-@cbook.deprecated("3.3")
-def validate_hinting(s):
-    return _validate_hinting(s)
-
-
-# Replace by plain list in _prop_validators after deprecation period.
-def _validate_hinting(s):
-    if s in (True, False):
-        cbook.warn_deprecated(
-            "3.2", message="Support for setting the text.hinting rcParam to "
-            "True or False is deprecated since %(since)s and will be removed "
-            "%(removal)s; set it to its synonyms 'auto' or 'none' instead.")
-        return s
-    return ValidateInStrings(
-        'text.hinting',
-        ['default', 'no_autohint', 'force_autohint', 'no_hinting',
-         'auto', 'native', 'either', 'none'],
-        ignorecase=True)(s)
-
-
-validate_pgf_texsystem = ValidateInStrings(
-    'pgf.texsystem', ['xelatex', 'lualatex', 'pdflatex'],
-    _deprecated_since="3.3")
-
-
-@cbook.deprecated("3.3")
-def validate_movie_writer(s):
-    # writers.list() would only list actually available writers, but
-    # FFMpeg.isAvailable is slow and not worth paying for at every import.
-    if s in animation.writers._registered:
-        return s
-    else:
-        raise ValueError(f"Supported animation writers are "
-                         f"{sorted(animation.writers._registered)}")
-
-
-validate_movie_frame_fmt = ValidateInStrings(
-    'animation.frame_format', ['png', 'jpeg', 'tiff', 'raw', 'rgba'],
-    _deprecated_since="3.3")
-validate_axis_locator = ValidateInStrings(
-    'major', ['minor', 'both', 'major'], _deprecated_since="3.3")
-validate_movie_html_fmt = ValidateInStrings(
-    'animation.html', ['html5', 'jshtml', 'none'], _deprecated_since="3.3")
-
 
 def validate_bbox(s):
     if isinstance(s, str):
@@ -784,10 +574,6 @@ _range_validators = {  # Slightly nicer (internal) API.
 }
 
 
-validate_grid_axis = ValidateInStrings(
-    'axes.grid.axis', ['x', 'y', 'both'], _deprecated_since="3.3")
-
-
 def validate_hatch(s):
     r"""
     Validate a hatch pattern.
@@ -796,7 +582,7 @@ def validate_hatch(s):
     """
     if not isinstance(s, str):
         raise ValueError("Hatch pattern must be a string")
-    cbook._check_isinstance(str, hatch_pattern=s)
+    _api.check_isinstance(str, hatch_pattern=s)
     unknown = set(s) - {'\\', '/', '|', '-', '+', '*', '.', 'x', 'o', 'O'}
     if unknown:
         raise ValueError("Unknown hatch symbol(s): %s" % list(unknown))
@@ -814,8 +600,8 @@ _prop_validators = {
         'linestyle': _listify_validator(_validate_linestyle),
         'facecolor': validate_colorlist,
         'edgecolor': validate_colorlist,
-        'joinstyle': validate_joinstylelist,
-        'capstyle': validate_capstylelist,
+        'joinstyle': _listify_validator(JoinStyle),
+        'capstyle': _listify_validator(CapStyle),
         'fillstyle': validate_fillstylelist,
         'markerfacecolor': validate_colorlist,
         'markersize': validate_floatlist,
@@ -902,7 +688,7 @@ def cycler(*args, **kwargs):
     if len(args) == 1:
         if not isinstance(args[0], Cycler):
             raise TypeError("If only one positional argument given, it must "
-                            " be a Cycler instance.")
+                            "be a Cycler instance.")
         return validate_cycler(args[0])
     elif len(args) == 2:
         pairs = [(args[0], args[1])]
@@ -925,6 +711,13 @@ def cycler(*args, **kwargs):
     return reduce(operator.add, (ccycler(k, v) for k, v in validated))
 
 
+class _DunderChecker(ast.NodeVisitor):
+    def visit_Attribute(self, node):
+        if node.attr.startswith("__") and node.attr.endswith("__"):
+            raise ValueError("cycler strings with dunders are forbidden")
+        self.generic_visit(node)
+
+
 def validate_cycler(s):
     """Return a Cycler object from a string repr or the object itself."""
     if isinstance(s, str):
@@ -936,23 +729,21 @@ def validate_cycler(s):
         # I locked it down by only having the 'cycler()' function available.
         # UPDATE: Partly plugging a security hole.
         # I really should have read this:
-        # http://nedbatchelder.com/blog/201206/eval_really_is_dangerous.html
+        # https://nedbatchelder.com/blog/201206/eval_really_is_dangerous.html
         # We should replace this eval with a combo of PyParsing and
         # ast.literal_eval()
         try:
-            if '.__' in s.replace(' ', ''):
-                raise ValueError("'%s' seems to have dunder methods. Raising"
-                                 " an exception for your safety")
+            _DunderChecker().visit(ast.parse(s))
             s = eval(s, {'cycler': cycler, '__builtins__': {}})
         except BaseException as e:
-            raise ValueError("'%s' is not a valid cycler construction: %s" %
-                             (s, e)) from e
+            raise ValueError(f"{s!r} is not a valid cycler construction: {e}"
+                             ) from e
     # Should make sure what comes from the above eval()
     # is a Cycler object.
     if isinstance(s, Cycler):
         cycler_inst = s
     else:
-        raise ValueError("object was not a string or Cycler instance: %s" % s)
+        raise ValueError(f"Object is not a string or Cycler instance: {s!r}")
 
     unknowns = cycler_inst.keys - (set(_prop_validators) | set(_prop_aliases))
     if unknowns:
@@ -964,12 +755,11 @@ def validate_cycler(s):
     for prop in cycler_inst.keys:
         norm_prop = _prop_aliases.get(prop, prop)
         if norm_prop != prop and norm_prop in cycler_inst.keys:
-            raise ValueError("Cannot specify both '{0}' and alias '{1}'"
-                             " in the same prop_cycle".format(norm_prop, prop))
+            raise ValueError(f"Cannot specify both {norm_prop!r} and alias "
+                             f"{prop!r} in the same prop_cycle")
         if norm_prop in checker:
-            raise ValueError("Another property was already aliased to '{0}'."
-                             " Collision normalizing '{1}'.".format(norm_prop,
-                                                                    prop))
+            raise ValueError(f"Another property was already aliased to "
+                             f"{norm_prop!r}. Collision normalizing {prop!r}.")
         checker.update([norm_prop])
 
     # This is just an extra-careful check, just in case there is some
@@ -1003,39 +793,6 @@ def validate_hist_bins(s):
                      " a sequence of floats".format(valid_strs))
 
 
-@cbook.deprecated("3.2")
-def validate_animation_writer_path(p):
-    # Make sure it's a string and then figure out if the animations
-    # are already loaded and reset the writers (which will validate
-    # the path on next call)
-    cbook._check_isinstance(str, path=p)
-    from sys import modules
-    # set dirty, so that the next call to the registry will re-evaluate
-    # the state.
-    # only set dirty if already loaded. If not loaded, the load will
-    # trigger the checks.
-    if "matplotlib.animation" in modules:
-        modules["matplotlib.animation"].writers.set_dirty()
-    return p
-
-
-@cbook.deprecated("3.3")
-def validate_webagg_address(s):
-    if s is not None:
-        import socket
-        try:
-            socket.inet_aton(s)
-        except socket.error as e:
-            raise ValueError(
-                "'webagg.address' is not a valid IP address") from e
-        return s
-    raise ValueError("'webagg.address' is not a valid IP address")
-
-
-validate_axes_titlelocation = ValidateInStrings(
-    'axes.titlelocation', ['left', 'center', 'right'], _deprecated_since="3.3")
-
-
 class _ignorecase(list):
     """A marker class indicating that a list-of-str is case-insensitive."""
 
@@ -1056,8 +813,8 @@ def _convert_validator_spec(key, conv):
 _validators = {
     "backend":           validate_backend,
     "backend_fallback":  validate_bool,
-    "toolbar":           _ignorecase(["none", "toolbar2", "toolmanager"]),
-    "datapath":          validate_any,  # see _get_data_path_cached
+    "figure.hooks":      validate_stringlist,
+    "toolbar":           _validate_toolbar,
     "interactive":       validate_bool,
     "timezone":          validate_string,
 
@@ -1076,10 +833,10 @@ _validators = {
     "lines.markeredgewidth": validate_float,
     "lines.markersize":      validate_float,  # markersize, in points
     "lines.antialiased":     validate_bool,  # antialiased (no jaggies)
-    "lines.dash_joinstyle":  validate_joinstyle,
-    "lines.solid_joinstyle": validate_joinstyle,
-    "lines.dash_capstyle":   validate_capstyle,
-    "lines.solid_capstyle":  validate_capstyle,
+    "lines.dash_joinstyle":  JoinStyle,
+    "lines.solid_joinstyle": JoinStyle,
+    "lines.dash_capstyle":   CapStyle,
+    "lines.solid_capstyle":  CapStyle,
     "lines.dashed_pattern":  validate_floatlist,
     "lines.dashdot_pattern": validate_floatlist,
     "lines.dotted_pattern":  validate_floatlist,
@@ -1090,6 +847,7 @@ _validators = {
 
     ## pcolor(mesh) props:
     "pcolor.shading": ["auto", "flat", "nearest", "gouraud"],
+    "pcolormesh.snap": validate_bool,
 
     ## patch props
     "patch.linewidth":       validate_float,  # line width in points
@@ -1154,7 +912,7 @@ _validators = {
     "font.family":     validate_stringlist,  # used by text object
     "font.style":      validate_string,
     "font.variant":    validate_string,
-    "font.stretch":    validate_string,
+    "font.stretch":    validate_fontstretch,
     "font.weight":     validate_fontweight,
     "font.size":       validate_float,  # Base font size in points
     "font.serif":      validate_stringlist,
@@ -1166,12 +924,13 @@ _validators = {
     # text props
     "text.color":          validate_color,
     "text.usetex":         validate_bool,
-    "text.latex.preamble": _validate_tex_preamble,
-    "text.latex.preview":  validate_bool,
-    "text.hinting":        _validate_hinting,
+    "text.latex.preamble": validate_string,
+    "text.hinting":        ["default", "no_autohint", "force_autohint",
+                            "no_hinting", "auto", "native", "either", "none"],
     "text.hinting_factor": validate_int,
     "text.kerning_factor": validate_int,
     "text.antialiased":    validate_bool,
+    "text.parse_math":     validate_bool,
 
     "mathtext.cal":            validate_font_properties,
     "mathtext.rm":             validate_font_properties,
@@ -1183,12 +942,11 @@ _validators = {
                                 "stixsans", "custom"],
     "mathtext.default":        ["rm", "cal", "it", "tt", "sf", "bf", "default",
                                 "bb", "frak", "scr", "regular"],
-    "mathtext.fallback_to_cm": _validate_mathtext_fallback_to_cm,
     "mathtext.fallback":       _validate_mathtext_fallback,
 
     "image.aspect":          validate_aspect,  # equal, auto, a number
     "image.interpolation":   validate_string,
-    "image.cmap":            validate_string,  # gray, jet, etc.
+    "image.cmap":            _validate_cmap,  # gray, jet, etc.
     "image.lut":             validate_int,  # lookup table
     "image.origin":          ["upper", "lower"],
     "image.resample":        validate_bool,
@@ -1200,6 +958,7 @@ _validators = {
     "contour.negative_linestyle": _validate_linestyle,
     "contour.corner_mask":        validate_bool,
     "contour.linewidth":          validate_float_or_None,
+    "contour.algorithm":          ["mpl2005", "mpl2014", "serial", "threaded"],
 
     # errorbar props
     "errorbar.capsize": validate_float,
@@ -1254,10 +1013,15 @@ _validators = {
     "axes.autolimit_mode": ["data", "round_numbers"],
     "axes.xmargin": _range_validators["0 <= x <= 1"],  # margin added to xaxis
     "axes.ymargin": _range_validators["0 <= x <= 1"],  # margin added to yaxis
+    'axes.zmargin': _range_validators["0 <= x <= 1"],  # margin added to zaxis
 
     "polaraxes.grid": validate_bool,  # display polar grid or not
     "axes3d.grid":    validate_bool,  # display 3d grid
 
+    "axes3d.xaxis.panecolor":    validate_color,  # 3d background pane
+    "axes3d.yaxis.panecolor":    validate_color,  # 3d background pane
+    "axes3d.zaxis.panecolor":    validate_color,  # 3d background pane
+
     # scatter props
     "scatter.marker":     validate_string,
     "scatter.edgecolors": validate_string,
@@ -1271,6 +1035,10 @@ _validators = {
     "date.autoformatter.second":      validate_string,
     "date.autoformatter.microsecond": validate_string,
 
+    'date.converter':          ['auto', 'concise'],
+    # for auto date locator, choose interval_multiples
+    'date.interval_multiples': validate_bool,
+
     # legend properties
     "legend.fancybox": validate_bool,
     "legend.loc": _ignorecase([
@@ -1278,18 +1046,21 @@ _validators = {
         "upper right", "upper left", "lower left", "lower right", "right",
         "center left", "center right", "lower center", "upper center",
         "center"]),
+
     # the number of points in the legend line
     "legend.numpoints":      validate_int,
     # the number of points in the legend line for scatter
     "legend.scatterpoints":  validate_int,
     "legend.fontsize":       validate_fontsize,
     "legend.title_fontsize": validate_fontsize_None,
-     # the relative size of legend markers vs. original
+    # color of the legend
+    "legend.labelcolor":     _validate_color_or_linecolor,
+    # the relative size of legend markers vs. original
     "legend.markerscale":    validate_float,
     "legend.shadow":         validate_bool,
-     # whether or not to draw a frame around legend
+    # whether or not to draw a frame around legend
     "legend.frameon":        validate_bool,
-     # alpha value of the legend frame
+    # alpha value of the legend frame
     "legend.framealpha":     validate_float_or_None,
 
     ## the following dimensions are in fraction of the font size
@@ -1320,14 +1091,15 @@ _validators = {
     "xtick.minor.width":   validate_float,     # minor xtick width in points
     "xtick.major.pad":     validate_float,     # distance to label in points
     "xtick.minor.pad":     validate_float,     # distance to label in points
-    "xtick.color":         validate_color,     # color of xtick labels
+    "xtick.color":         validate_color,     # color of xticks
+    "xtick.labelcolor":    validate_color_or_inherit,  # color of xtick labels
     "xtick.minor.visible": validate_bool,      # visibility of minor xticks
     "xtick.minor.top":     validate_bool,      # draw top minor xticks
     "xtick.minor.bottom":  validate_bool,      # draw bottom minor xticks
     "xtick.major.top":     validate_bool,      # draw top major xticks
     "xtick.major.bottom":  validate_bool,      # draw bottom major xticks
     "xtick.labelsize":     validate_fontsize,  # fontsize of xtick labels
-    "xtick.direction":     validate_string,    # direction of xticks
+    "xtick.direction":     ["out", "in", "inout"],  # direction of xticks
     "xtick.alignment":     ["center", "right", "left"],
 
     "ytick.left":          validate_bool,      # draw ticks on left side
@@ -1340,14 +1112,15 @@ _validators = {
     "ytick.minor.width":   validate_float,     # minor ytick width in points
     "ytick.major.pad":     validate_float,     # distance to label in points
     "ytick.minor.pad":     validate_float,     # distance to label in points
-    "ytick.color":         validate_color,     # color of ytick labels
+    "ytick.color":         validate_color,     # color of yticks
+    "ytick.labelcolor":    validate_color_or_inherit,  # color of ytick labels
     "ytick.minor.visible": validate_bool,      # visibility of minor yticks
     "ytick.minor.left":    validate_bool,      # draw left minor yticks
     "ytick.minor.right":   validate_bool,      # draw right minor yticks
     "ytick.major.left":    validate_bool,      # draw left major yticks
     "ytick.major.right":   validate_bool,      # draw right major yticks
     "ytick.labelsize":     validate_fontsize,  # fontsize of ytick labels
-    "ytick.direction":     validate_string,    # direction of yticks
+    "ytick.direction":     ["out", "in", "inout"],  # direction of yticks
     "ytick.alignment":     [
         "center", "top", "bottom", "baseline", "center_baseline"],
 
@@ -1361,6 +1134,10 @@ _validators = {
     "figure.titlesize":   validate_fontsize,
     "figure.titleweight": validate_fontweight,
 
+    # figure labels
+    "figure.labelsize":   validate_fontsize,
+    "figure.labelweight": validate_fontweight,
+
     # figure size in inches: width by height
     "figure.figsize":          _listify_validator(validate_float, n=2),
     "figure.dpi":              validate_float,
@@ -1392,12 +1169,11 @@ _validators = {
     'savefig.facecolor':    validate_color_or_auto,
     'savefig.edgecolor':    validate_color_or_auto,
     'savefig.orientation':  ['landscape', 'portrait'],
-    'savefig.jpeg_quality': validate_int,
-    "savefig.format":       _update_savefig_format,
+    "savefig.format":       validate_string,
     "savefig.bbox":         validate_bbox,  # "tight", or "standard" (= None)
     "savefig.pad_inches":   validate_float,
     # default directory in savefig dialog box
-    "savefig.directory":    validate_string,
+    "savefig.directory":    _validate_pathlike,
     "savefig.transparent":  validate_bool,
 
     "tk.window_focus": validate_bool,  # Maintain shell focus for TkAgg
@@ -1419,7 +1195,7 @@ _validators = {
 
     "pgf.texsystem": ["xelatex", "lualatex", "pdflatex"],  # latex variant used
     "pgf.rcfonts":   validate_bool,  # use mpl's rc settings for font config
-    "pgf.preamble":  _validate_tex_preamble,  # custom LaTeX preamble
+    "pgf.preamble":  validate_string,  # custom LaTeX preamble
 
     # write raster image data into the svg file
     "svg.image_inline": validate_bool,
@@ -1450,7 +1226,6 @@ _validators = {
     "keymap.grid_minor": validate_stringlist,
     "keymap.yscale":     validate_stringlist,
     "keymap.xscale":     validate_stringlist,
-    "keymap.all_axes":   validate_stringlist,
     "keymap.help":       validate_stringlist,
     "keymap.copy":       validate_stringlist,
 
@@ -1463,24 +1238,17 @@ _validators = {
     "animation.codec":        validate_string,
     "animation.bitrate":      validate_int,
     # Controls image format when frames are written to disk
-    "animation.frame_format": ["png", "jpeg", "tiff", "raw", "rgba"],
-    # Additional arguments for HTML writer
-    "animation.html_args":    validate_stringlist,
+    "animation.frame_format": ["png", "jpeg", "tiff", "raw", "rgba", "ppm",
+                               "sgi", "bmp", "pbm", "svg"],
     # Path to ffmpeg binary. If just binary name, subprocess uses $PATH.
-    "animation.ffmpeg_path":  validate_string,
+    "animation.ffmpeg_path":  _validate_pathlike,
     # Additional arguments for ffmpeg movie writer (using pipes)
     "animation.ffmpeg_args":  validate_stringlist,
-    # Path to AVConv binary. If just binary name, subprocess uses $PATH.
-    "animation.avconv_path":  validate_string,
-    # Additional arguments for avconv movie writer (using pipes)
-    "animation.avconv_args":  validate_stringlist,
      # Path to convert binary. If just binary name, subprocess uses $PATH.
-    "animation.convert_path": validate_string,
+    "animation.convert_path": _validate_pathlike,
      # Additional arguments for convert movie writer (using pipes)
     "animation.convert_args": validate_stringlist,
 
-    "mpl_toolkits.legacy_colorbar": validate_bool,
-
     # Classic (pre 2.0) compatibility mode
     # This is used for things that are hard to make backward compatible
     # with a sane rcParam alone.  This does *not* turn on classic mode
@@ -1488,18 +1256,11 @@ _validators = {
     "_internal.classic_mode": validate_bool
 }
 _hardcoded_defaults = {  # Defaults not inferred from matplotlibrc.template...
-    # ... because it can"t be:
-    "backend": _auto_backend_sentinel,
     # ... because they are private:
     "_internal.classic_mode": False,
     # ... because they are deprecated:
-    "animation.avconv_path": "avconv",
-    "animation.avconv_args": [],
-    "animation.html_args": [],
-    "mathtext.fallback_to_cm": None,
-    "keymap.all_axes": ["a"],
-    "savefig.jpeg_quality": 95,
-    "text.latex.preview": False,
+    # No current deprecations.
+    # backend is handled separately when constructing rcParamsDefault.
 }
 _validators = {k: _convert_validator_spec(k, conv)
                for k, conv in _validators.items()}
